package com.bsz.school_send_back_end.aop;

import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.enums.UserRoleEnum;
import com.bsz.school_send_back_end.service.UserService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 权限校验 AOP
 */
@Aspect
@Component
public class AuthInterceptor {

    @Resource
    private UserService userService;

    /**
     * 执行拦截
     */
    @Around("@annotation(authCheck)")
    public Object doInterceptor(ProceedingJoinPoint joinPoint, AuthCheck authCheck) throws Throwable {

        String  mustRole = authCheck.mustRole();
        RequestAttributes requestAttributes = RequestContextHolder.currentRequestAttributes();
        HttpServletRequest request = ((ServletRequestAttributes) requestAttributes).getRequest();
        String uri = request.getRequestURI();
        // ✅ 如果是 Swagger 或 OpenAPI 相关请求，直接放行
        if (uri.startsWith("/v3/api-docs") || uri.contains("/swagger") || uri.contains("/swagger-ui")) {
            return joinPoint.proceed();  // 直接放行
        }
        // 当前登录用户
        User loginUser = userService.getLoginUser(request);
        // 必须有该权限才通过
        UserRoleEnum mustUserRoleEnum = UserRoleEnum.getEnumByValue(mustRole);

        String userRole = loginUser.getUserRole();
        // 如果被封号，直接拒绝
        if (UserRoleEnum.BAN.equals(UserRoleEnum.getEnumByValue(userRole))) {
            throw new BusinessException(ErrorCode.NO_AUTH);
        }

        // 必须有Boss权限
        if (UserRoleEnum.BOSS.equals(mustUserRoleEnum)) {
            if (!mustRole.equals(userRole)) {
                throw new BusinessException(ErrorCode.NO_AUTH);
            }
        }

        // 必须有管理员权限
        if (UserRoleEnum.ADMIN.equals(mustUserRoleEnum)) {
            if (!mustRole.equals(userRole) && !UserConstant.BOSS_ROLE.equals(userRole)) {
                throw new BusinessException(ErrorCode.NO_AUTH);
            }
        }
        // 必须有商家或管理员权限
        if (UserRoleEnum.BUSINESS.equals(mustUserRoleEnum)) {
            if (!Arrays.asList(mustRole, UserConstant.ADMIN_ROLE, UserConstant.BOSS_ROLE).contains(userRole)) {
                throw new BusinessException(ErrorCode.NO_AUTH);
            }
        }

        //
        if (UserRoleEnum.ERRAND.equals(mustUserRoleEnum)) {
            if (!Arrays.asList(mustRole, UserConstant.ADMIN_ROLE, UserConstant.BOSS_ROLE).contains(userRole)) {
                throw new BusinessException(ErrorCode.NO_AUTH);
            }
        }

        if (UserRoleEnum.USER.equals(mustUserRoleEnum)) {
            if (!Arrays.asList(mustRole, UserConstant.ADMIN_ROLE, UserConstant.BOSS_ROLE).contains(userRole)) {
                throw new BusinessException(ErrorCode.NO_AUTH);
            }
        }
        // 通过权限校验，放行
        return joinPoint.proceed();
    }
}
