package com.bsz.school_send_back_end.aop;


import com.bsz.school_send_back_end.annotation.SystemLog;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Systemlog;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.service.SystemlogService;
import com.bsz.school_send_back_end.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;

/**
 * 日志AOP
 */
@Slf4j
@Aspect
@Component
public class SystemLogInterceptor {

    @Resource
    private UserService userService;

    @Resource
    private SystemlogService systemlogService;

    /**
     * 执行拦截
     */
    @Around("@annotation(systemLog)")
    public Object doInterceptor(ProceedingJoinPoint joinPoint, SystemLog systemLog) throws  Throwable {

        // 获取执行操作
        String executeStep = systemLog.executeStep();
        ThrowUtils.throwIf(StringUtils.isEmpty(executeStep), ErrorCode.PARAMS_ERROR);
        //获取请求
        RequestAttributes requestAttributes = RequestContextHolder.currentRequestAttributes();
        HttpServletRequest httpServletRequest = ((ServletRequestAttributes) requestAttributes).getRequest();
        String uri = httpServletRequest.getRequestURI();
        // ✅ 如果是 Swagger 或 OpenAPI 相关请求，直接放行
        if (uri.startsWith("/v3/api-docs") || uri.contains("/swagger") || uri.contains("/swagger-ui")) {
            return joinPoint.proceed();  // 直接放行
        }
        //获取登录用户
        User loginUser = userService.getLoginUser(httpServletRequest);
        //写入系统日志
        Systemlog systemlog = new Systemlog();
        systemlog.setUserId(loginUser.getId());
        systemlog.setContent(executeStep);
        systemlog.setIp(httpServletRequest.getRemoteHost());
        boolean save = systemlogService.save(systemlog);
        if (!save) {
            log.error("操作日志写入失败，执行操作：" + executeStep);
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "操作日志写入失败");
        }

        // 放行
        return joinPoint.proceed();
    }

}
