package com.bsz.school_send_back_end.config;

import com.bsz.school_send_back_end.model.domain.Dishes;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.vo.BusinessVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.DishesService;
import com.bsz.school_send_back_end.service.LoggingService;
import com.bsz.school_send_back_end.service.OrdersService;
import jakarta.annotation.Resource;
import org.springframework.ai.chat.client.ChatClient;
import org.springframework.ai.chat.client.advisor.PromptChatMemoryAdvisor;
import org.springframework.ai.chat.memory.ChatMemory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Description;

import java.util.List;
import java.util.function.Function;

@Configuration
public class AIConfig {

    @Resource
    private BusinessService businessService;

    @Resource
    private DishesService dishesService;

    @Resource
    private OrdersService ordersService;

    @Bean
    ChatClient chatClient(ChatClient.Builder builder, ChatMemory chatMemory) {
        return builder.defaultSystem(
                """
                        你是食刻必达校快送小程序的助手，情谊轻松切愉快的语气与用户交流。
                        你可以给用户推荐一些商家和菜品
                        今天的日期是{current_date}。
                        请讲中文。
                        """
                )
                .defaultAdvisors(
                        new PromptChatMemoryAdvisor(chatMemory),
                        new LoggingService()
                )
                .defaultFunctions("getBusinessByAI")
                .build();
    }


    @Bean
    @Description("推荐商家")
    public Function<Void, List<BusinessVO>> getBusinessByAI() {
        List<BusinessVO> allBusiness = businessService.findAllBusiness();
        return unused -> allBusiness.stream()
                .filter(businessVO -> businessVO.getLevel() == 5)
                .toList();
    }

//    @Bean
//    @Description("推荐菜品")
//    public Function<Void, List<Dishes>> getDishesByAI() {
//        new LocalTime
//    }
}
