package com.bsz.school_send_back_end.config;

import com.bsz.school_send_back_end.contant.RabbitMQConstant;
import org.springframework.amqp.core.*;
import org.springframework.amqp.support.converter.DefaultClassMapper;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.Map;

@Configuration
public class RabbitMQConfig {

    @Bean
    public Queue topicQueue1() {
        return QueueBuilder.durable(RabbitMQConstant.TOPIC_QUEUE1).build();
    }

    @Bean
    public Queue topicQueue2() {
        return QueueBuilder.durable(RabbitMQConstant.TOPIC_QUEUE2).build();
    }
    @Bean
    public Queue orderReplyQueue() {
        return QueueBuilder.durable(RabbitMQConstant.ORDER_REPLY_QUEUE).build();
    }

    @Bean
    public Exchange topicExchange() {
        return ExchangeBuilder.topicExchange(RabbitMQConstant.TOPIC_EXCHANGE).build();
    }

    @Bean
    public Exchange responseExchange() {
        return ExchangeBuilder.topicExchange(RabbitMQConstant.RESPONSE_EXCHANGE).build();
    }

    @Bean
    public Binding topicBinding1() {
        return BindingBuilder.bind(topicQueue1()).to(topicExchange()).with("topic.key1").noargs();
    }

    @Bean
    public Binding topicBinding2() {
        return BindingBuilder.bind(topicQueue2()).to(topicExchange()).with("topic.#").noargs();
    }

    @Bean
    public Binding responseBinding() {
        return BindingBuilder.bind(orderReplyQueue()).to(responseExchange()).with("response.#").noargs();
    }



//    /**
//     * 声明一个 **延迟交换机**，类型必须为 `x-delayed-message`
//     */
//    @Bean
//    public CustomExchange delayedExchange() {
//        Map<String, Object> args = new HashMap<>();
//        args.put("x-delayed-type", "direct"); // 交换机类型
//        return new CustomExchange(RabbitMQConstant.DELAYED_EXCHANGE, "x-delayed-message", true, false, args);
//    }
//
//    /**
//     * 声明队列
//     */
//    @Bean
//    public Queue delayedQueue() {
//        return new Queue(RabbitMQConstant.DELAYED_QUEUE, true);
//    }
//
//    /**
//     * 绑定队列到延迟交换机
//     */
//    @Bean
//    public Binding binding(Queue delayedQueue, CustomExchange delayedExchange) {
//        return BindingBuilder.bind(delayedQueue).to(delayedExchange).with(RabbitMQConstant.DELAYED_ROUTING_KEY).noargs();
//    }

    @Bean
    public MessageConverter jsonToMapMessageConverter() {
        DefaultClassMapper defaultClassMapper = new DefaultClassMapper();
        defaultClassMapper.setTrustedPackages("com.bsz.school_send_back_end.utils.MultiDelayMessage"); // trusted packages
        Jackson2JsonMessageConverter jackson2JsonMessageConverter = new Jackson2JsonMessageConverter();
        jackson2JsonMessageConverter.setClassMapper(defaultClassMapper);
        return jackson2JsonMessageConverter;
    }

    @Bean
    public MessageConverter messageConverter(){
        return new Jackson2JsonMessageConverter();
    }

}
