package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.model.domain.Errand;
import com.bsz.school_send_back_end.model.domain.ErrandBill;
import com.bsz.school_send_back_end.model.domain.OrderRefunds;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.service.ErrandBillService;
import com.bsz.school_send_back_end.service.ErrandService;
import com.bsz.school_send_back_end.service.OrdersService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;

import java.util.*;
import java.util.stream.Collectors;

@Tag(name = "跑腿流水接口")
@RestController
@RequestMapping("/bill")
@Slf4j
public class BillController {

    @Resource
    private ErrandService errandService;

    @Resource
    private ErrandBillService billService;

    @Resource
    private OrdersService ordersService;

    /**
     * 跑腿查询自己的账单
     * @param request 网路请求
     * @return 账单列表
     */
    @PostMapping("/list")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<List<ErrandBill>> listBillByErrand(HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);
        LambdaQueryWrapper<Orders> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Orders::getErrandId, loginErrand.getId());
        List<Orders> ordersList = ordersService.list(wrapper);

        // 如果订单列表为空，直接返回空结果
        if (ordersList.isEmpty()) {
            return ResultUtils.success(Collections.emptyList());
        }
        List<Long> orderIds = ordersList.stream()
                .map(Orders::getId)
                .collect(Collectors.toList());
        LambdaQueryWrapper<ErrandBill> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(ErrandBill::getOrderId, orderIds);
        queryWrapper.orderByDesc(ErrandBill::getOrderStartTime);// 按 orderStartTime 降序排序

        List<ErrandBill> bills = billService.list(queryWrapper);

        return ResultUtils.success(bills);
    }


}
