package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Collect;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.CommonRequest;
import com.bsz.school_send_back_end.model.dto.collect.CollectAddRequest;
import com.bsz.school_send_back_end.model.vo.CollectVO;
import com.bsz.school_send_back_end.service.CollectService;
import com.bsz.school_send_back_end.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

@Tag(name = "收藏接口")
@RestController
@Slf4j
@RequestMapping("/collect")
public class CollectController {

    @Resource
    private CollectService collectService;

    @Resource
    private UserService userService;

    /**
     * 添加收藏
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addCollect(@RequestBody CollectAddRequest collectAddRequest, HttpServletRequest request) {
        if (collectAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        collectService.validCollect(collectAddRequest);
        Collect collect = new Collect();
        BeanUtils.copyProperties(collectAddRequest, collect);
        Boolean result = collectService.addCollect(collect, request);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 删除一条收藏
     */
    @PostMapping("/delete")
    public BaseResponse<Boolean> deleteCollect(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        LambdaQueryWrapper<Collect> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Collect::getUserId, userId);
        wrapper.eq(Collect::getBusinessId, id);
        boolean remove = collectService.remove(wrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.SYSTEM_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 删除所有收藏
     */
    @PostMapping("/delete/all")
    public BaseResponse<Boolean> deleteCollectAll(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long loginUserId = loginUser.getId();
        LambdaQueryWrapper<Collect> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Collect::getUserId, loginUserId);
        boolean remove = collectService.remove(wrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.SYSTEM_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 查询所有收藏
     */
    @PostMapping("/list")
    public BaseResponse<List<CollectVO>> listCollect(HttpServletRequest request) {
        List<CollectVO> collectList = collectService.getCollectList(request);

        if (collectList == null) {
            collectList = new ArrayList<>();
        }
        return ResultUtils.success(collectList);
    }
}
