package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Errand;
import com.bsz.school_send_back_end.model.domain.ErrandAuth;
import com.bsz.school_send_back_end.model.dto.errandAtuh.ErrandAuthAddRequest;
import com.bsz.school_send_back_end.model.dto.errandAtuh.ErrandAuthQueryRequest;
import com.bsz.school_send_back_end.service.ErrandAuthService;
import com.bsz.school_send_back_end.service.ErrandService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;

@Tag(name = "快送员认证接口")
@Slf4j
@RestController
@RequestMapping("/errandAuth")
public class ErrandAuthController {

    @Resource
    private ErrandAuthService errandAuthService;

    @Resource
    private ErrandService errandService;

    @PostMapping("/add")
    public BaseResponse<Boolean> addErrandAuth(@RequestBody ErrandAuthAddRequest addRequest) {
        if (addRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long errandId = addRequest.getErrandId();
        LambdaQueryWrapper<ErrandAuth> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ErrandAuth::getErrandId, errandId);
        long count = errandAuthService.count(queryWrapper);
        ThrowUtils.throwIf(count > 0, ErrorCode.PARAMS_ERROR, "当前跑腿已认证");
        ErrandAuth errandAuth = new ErrandAuth();
        BeanUtils.copyProperties(addRequest, errandAuth);
        errandAuthService.validErrandAuth(errandAuth,true);

        boolean save = errandAuthService.save(errandAuth);
        ThrowUtils.throwIf(!save, ErrorCode.SYSTEM_ERROR, "添加跑腿认证失败");

        return ResultUtils.success(true);
    }

    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<Boolean> updateErrandAuth (@RequestBody ErrandAuthAddRequest updateRequest) {
        if (updateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        ErrandAuth errandAuth = new ErrandAuth();
        BeanUtils.copyProperties(updateRequest, errandAuth);
        errandAuthService.validErrandAuth(errandAuth, false);
        LambdaUpdateWrapper<ErrandAuth> wrapper = errandAuthService.getUpdateWrapper(errandAuth);
        boolean update = errandAuthService.update(wrapper);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR, "更新跑腿认证信息失败");

        return ResultUtils.success(true);
    }

    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<ErrandAuth>> listErrandAuthByPage(@RequestBody ErrandAuthQueryRequest errandAuthQueryRequest) {
        if (errandAuthQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long pageSize = errandAuthQueryRequest.getPageSize();
        long current = errandAuthQueryRequest.getCurrent();
        QueryWrapper<ErrandAuth> queryWrapper = errandAuthService.getQueryWrapper(errandAuthQueryRequest);
        Page<ErrandAuth> page = errandAuthService.page(new Page<>(current, pageSize),queryWrapper);

        return ResultUtils.success(page);
    }

    @GetMapping("/getErrandAuthById")
    public BaseResponse<ErrandAuth> getErrandAuthById (String errandId) {
        Long errandId1 = Long.valueOf(errandId);
        ErrandAuth errandAuth = errandAuthService.getErrandAuth(errandId1);

        return ResultUtils.success(errandAuth);
    }

    @PostMapping("/get/my")
    public BaseResponse<ErrandAuth> getMyErrandAuth (HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);
        Long id = loginErrand.getId();
        LambdaQueryWrapper<ErrandAuth> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ErrandAuth::getErrandId, id);
        ErrandAuth errandAuth = errandAuthService.getOne(queryWrapper);
        ThrowUtils.throwIf(errandAuth == null, ErrorCode.OPERATION_ERROR, "当前跑腿为认证");
        return ResultUtils.success(errandAuth);
    }
}
