package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Errand;
import com.bsz.school_send_back_end.model.domain.ErrandIncome;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.CommonRequest;
import com.bsz.school_send_back_end.model.dto.erradnIncome.ErrandIncomeQueryRequest;
import com.bsz.school_send_back_end.service.ErrandIncomeService;
import com.bsz.school_send_back_end.service.ErrandService;
import com.bsz.school_send_back_end.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Tag(name = "快送员收入接口")
@RestController
@Slf4j
@RequestMapping("/errandIncome")
public class ErrandIncomeController {

    @Resource
    private ErrandService errandService;

    @Resource
    private ErrandIncomeService errandIncomeService;

    @Resource
    private UserService userService;

    @PostMapping("/count/number")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<Long> countIncomeNumber(HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);

        LambdaQueryWrapper<ErrandIncome> queryWrapper = errandIncomeService.getQueryWrapper(loginErrand.getId());

        long count = errandIncomeService.count(queryWrapper);
        return ResultUtils.success(count);
    }

    @PostMapping("/count/money")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<BigDecimal> countIncomeMoney(HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);

        LambdaQueryWrapper<ErrandIncome> queryWrapper = errandIncomeService.getQueryWrapper(loginErrand.getId());
        queryWrapper.eq(ErrandIncome::getState, 1);
        List<ErrandIncome> list = errandIncomeService.list(queryWrapper);
        BigDecimal price = list.stream().map(ErrandIncome::getIncome).reduce(BigDecimal.ZERO, BigDecimal::add);

        return ResultUtils.success(price);
    }

    @PostMapping("/count/total-money")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<BigDecimal> countTotalIncome(HttpServletRequest request) {
        // 获取登录跑腿信息
        Errand loginErrand = errandService.getLoginErrand(request);

        // 从跑腿表直接获取总收入字段值
        BigDecimal totalIncome = loginErrand.getTotalPrice();

        // 返回结果
        return ResultUtils.success(totalIncome);
    }

    @PostMapping("/list")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<List<ErrandIncome>> listErrandIncome(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long errandId = commonRequest.getId();
        User loginUser = userService.getLoginUser(request);
        String userRole = loginUser.getUserRole();
        if (userRole.equals(UserConstant.ERRAND_ROLE)) {
            Errand loginErrand = errandService.getLoginErrand(request);
            Long id = loginErrand.getId();
            ThrowUtils.throwIf(!id.equals(errandId), ErrorCode.PARAMS_ERROR, "传输的跑腿id不正确");
        }
        LambdaQueryWrapper<ErrandIncome> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(ErrandIncome::getErrandId, errandId);
        List<ErrandIncome> list = errandIncomeService.list(queryWrapper);

        if (list == null) {
            list = new ArrayList<>();
        }
        return ResultUtils.success(list);
    }

    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<Page<ErrandIncome>> listErrandIncomeByPage(@RequestBody ErrandIncomeQueryRequest queryRequest) {
        if (queryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long pageSize = queryRequest.getPageSize();
        long current = queryRequest.getCurrent();
        QueryWrapper<ErrandIncome> queryWrapper = errandIncomeService.getWrapper(queryRequest);
        Page<ErrandIncome> page = errandIncomeService.page(new Page<>(current, pageSize), queryWrapper);
        if (page == null) {
            page = new Page<>();
        }

        return ResultUtils.success(page);
    }


    @PostMapping("/count/money/no")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<BigDecimal> countIncomeMoneyNo(HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);

        LambdaQueryWrapper<ErrandIncome> queryWrapper = errandIncomeService.getQueryWrapper(loginErrand.getId());
        queryWrapper.eq(ErrandIncome::getState, 0);
        List<ErrandIncome> list = errandIncomeService.list(queryWrapper);
        BigDecimal price = list.stream().map(ErrandIncome::getIncome).reduce(BigDecimal.ZERO, BigDecimal::add);

        return ResultUtils.success(price);
    }
}
