package com.bsz.school_send_back_end.controller;

import cn.hutool.core.io.FileUtil;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.FileConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.file.UploadFileRequest;
import com.bsz.school_send_back_end.model.enums.FileUploadBizEnum;
import com.bsz.school_send_back_end.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;

/**
 * 文件上传
 */
@Tag(name = "文件上传接口")
@Slf4j
@RestController
@RequestMapping("/file")
public class FileController {

    @Resource
    private UserService userService;
    @PostMapping("/upload/server/not_login")
    public BaseResponse<String> uploadServerNotLogin (@RequestPart("file")MultipartFile multipartFile, UploadFileRequest uploadFileRequest) {
        //获取业务名称
        String biz = uploadFileRequest.getBiz();
        FileUploadBizEnum fileUploadBizEnum = FileUploadBizEnum.getEnumByValue(biz);
        if (fileUploadBizEnum == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        //校验文件
        validfile(multipartFile, fileUploadBizEnum);
        //文件目录：根据业务，用户来划分
        String uuid = RandomStringUtils.randomAlphabetic(8);
        String fileName = uuid + "-" + multipartFile.getOriginalFilename();
        String filePath = String.format("/%s/%s/%s", fileUploadBizEnum.getValue(), 0, fileName);
        try {
            //判断目录是否存在
            File file = new File(FileConstant.SERVER_UPLOAD_DIR, filePath);
            log.info("filepath:{}",file);
            if (!file.exists()) {
                boolean mkdirs = file.mkdirs();
                if (!mkdirs) {
                    throw new BusinessException(ErrorCode.SYSTEM_ERROR,"创建目录失败");
                }
            }
            multipartFile.transferTo(new File(FileConstant.SERVER_UPLOAD_DIR, filePath));
            // 返回可访问地址
            return ResultUtils.success(FileConstant.SERVER_HOST + filePath);
        }
        catch (IOException e) {
            log.error("file upload error, filePath = " + filePath, e);
            throw new BusinessException(ErrorCode.SYSTEM_ERROR,"上传失败");
        }

    }

    @PostMapping("/upload/server")
    public BaseResponse<String> uploadServerFile(@RequestPart("file") MultipartFile multipartFile,  UploadFileRequest uploadFileRequest, HttpServletRequest request) {
        // 获取业务名称
        String biz = uploadFileRequest.getBiz();
        FileUploadBizEnum fileUploadBizEnum = FileUploadBizEnum.getEnumByValue(biz);
        if (fileUploadBizEnum == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "业务名称错误");
        }
        //校验文件
        validfile(multipartFile,fileUploadBizEnum);
        //校验用户是否登录
        User loginUser = userService.getLoginUser(request);
        if (loginUser == null) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "未登录");
        }
        // 文件目录：根据业务、用户来划分
        String uuid = RandomStringUtils.randomAlphabetic(8);
        String fileName = uuid + "-" + multipartFile.getOriginalFilename();
        String filePath = String.format("/%s/%s/%s", fileUploadBizEnum.getValue(), loginUser.getId(), fileName);
        try {
            File file = new File(FileConstant.SERVER_UPLOAD_DIR, filePath);
            if (!file.exists()) {
                //创建目录
                boolean mkdirs = file.mkdirs();

                if (!mkdirs) {
                    throw new BusinessException(ErrorCode.SYSTEM_ERROR, "创建目录失败");
                }
            }
            multipartFile.transferTo(new File(FileConstant.SERVER_UPLOAD_DIR,filePath));
            //返回可访问的地址
            return ResultUtils.success(FileConstant.SERVER_HOST + filePath);
        }
        catch (IOException e) {
            log.error("file upload error, filepath = " + filePath, e);
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "上传失败");
        }
    }

    /**
     * 校验文件
     */
    private void validfile(MultipartFile multipartFile, FileUploadBizEnum fileUploadBizEnum) {
        //文件大小
        long fileSize = multipartFile.getSize();
        //文件后缀
        String fileSuffix = FileUtil.getSuffix(multipartFile.getOriginalFilename());
        final long TWO_M = 2*1024*1024L;
        if (FileUploadBizEnum.USER_AVATAR.equals(fileUploadBizEnum)) {
            if (fileSize > TWO_M) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件大小不能超过 2M");
            }
            if (!Arrays.asList("jpeg", "jpg", "svg", "png", "webp").contains(fileSuffix)) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件类型错误");
            }
        }
    }


}