package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.PrivateMessage;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.service.PrivateMessageService;
import com.bsz.school_send_back_end.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Tag(name = "私聊")
@RestController
@RequestMapping("/PrivateMessage")
public class PrivateMessageController {

    @Resource
    private PrivateMessageService privateMessageService;

    @Resource
    private UserService userService;

    @Resource
    private SimpMessagingTemplate template;

    /**
     * 添加一条消息
     */
    @PostMapping("/save")
    public BaseResponse<Boolean> saveOneMessage (@RequestBody PrivateMessage message) {
        boolean save = privateMessageService.save(message);
        ThrowUtils.throwIf(!save, ErrorCode.SYSTEM_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 查询私聊记录
     */
    @GetMapping("/list/{fromUserAccount}/{toUserAccount}")
    public BaseResponse<List<String>> listPrivateMessage(
            @PathVariable("fromUserAccount") String fromUserAccount,
            @PathVariable("toUserAccount") String toUserAccount) {
        LambdaQueryWrapper<PrivateMessage> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.and(q -> q
                        .eq(PrivateMessage::getFrom_userAccount, fromUserAccount)
                        .eq(PrivateMessage::getTo_userAccount, toUserAccount)
                )
                .or(q -> q
                        .eq(PrivateMessage::getFrom_userAccount, toUserAccount)
                        .eq(PrivateMessage::getTo_userAccount, fromUserAccount)
                );
        List<PrivateMessage> list = privateMessageService.list(queryWrapper);
        List<String> collect = list.stream().map(PrivateMessage::getMessage).collect(Collectors.toList());

        return ResultUtils.success(collect);
    }

    /**
     * 删除所有私聊聊天记录
     */
    @PostMapping("/delete/{fromUserAccount}/{toUserAccount}")
    public BaseResponse<Boolean> deleteAllMessage(
            @PathVariable("fromUserAccount") String fromUserAccount,
            @PathVariable("toUserAccount") String toUserAccount) {

        // 更新当前用户的删除标志，不影响另一个用户的查看状态
        UpdateWrapper<PrivateMessage> updateWrapper = new UpdateWrapper<>();
        updateWrapper
                .and(q -> q.eq("from_userAccount", fromUserAccount)
                        .eq("to_userAccount", toUserAccount)
                        .set("from_user_deleted", 1)) // 更新发消息者的删除标志
                .or(q -> q.eq("from_userAccount", toUserAccount)
                        .eq("to_userAccount", fromUserAccount)
                        .set("to_user_deleted", 1)); // 更新接收消息者的删除标志

        boolean update = privateMessageService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR);

        // 只发送当前用户的删除消息通知，不发送给另一方
        User fromUser = userService.getOne(Wrappers.<User>lambdaQuery().eq(User::getUserAccount, fromUserAccount));
        template.convertAndSend("/topic/ServerToClient.deleteMsg", fromUser);

        return ResultUtils.success(true);
    }

}
