package com.bsz.school_send_back_end.controller;

import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.utils.AliyunSmsUtils;
import com.bsz.school_send_back_end.utils.RandomNumberGenerator;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

@Tag(name = "阿里云短信接口")
@RestController
@Slf4j
@RequestMapping("/sms")
public class SmsController {

    @Resource
    private RedisTemplate redisTemplate;
    
    @Resource
    private RandomNumberGenerator randomNumberGenerator;

    @Resource
    private AliyunSmsUtils aliyunSmsUtils;

    @GetMapping("/send/{phone}")
    public BaseResponse<String> sendCode (@PathVariable String phone) {
        String code = (String) redisTemplate.opsForValue().get(phone);
        log.info("获取验证码：{}", code);

        if (code == null) {
            String fourBitRandom = randomNumberGenerator.getFourBitRandom();
            HashMap<String, Object> hashMap = new HashMap<>();
            hashMap.put("code", fourBitRandom);
            log.info("生成的验证码：" + fourBitRandom);
            redisTemplate.opsForValue().set(phone, fourBitRandom, 5, TimeUnit.MINUTES);
            boolean b = aliyunSmsUtils.sendSms(phone, hashMap);
            log.info("短信发送状态,{}", b);
            return ResultUtils.success("发送成功");
        } else {
            return ResultUtils.success("请勿重复发送验证码");
        }
    }

}
