package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Systemlog;
import com.bsz.school_send_back_end.model.dto.log.LogQueryRequest;
import com.bsz.school_send_back_end.model.vo.BusinessLogVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.SystemlogService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Tag(name = "系统日志接口")
@RestController
@RequestMapping("/log")
@Slf4j
public class SystemLogController {

    @Resource
    private BusinessService businessService;

    @Resource
    private SystemlogService systemlogService;
    /**
     * 分页获取商家系统日志列表
     */
    @PostMapping("/business/logs")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<BusinessLogVO>> getLogByPage(@RequestBody LogQueryRequest logQueryRequest) {
        if (logQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = logQueryRequest.getCurrent();
        long pageSize = logQueryRequest.getPageSize();

        //根据手机号获取用户id
        String businessPhone = logQueryRequest.getBusinessPhone();
        if (StringUtils.isNotBlank(businessPhone)) {
            LambdaQueryWrapper<Business> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(Business::getBusinessPhone, businessPhone);
            Business business = businessService.getOne(wrapper);
            if (business == null) {
                logQueryRequest.setUserId((long) -1);
            }
            else {
                logQueryRequest.setUserId(business.getUserId());
            }
        }
        Page<Systemlog> systemlogPage = systemlogService.page(new Page<>(current, pageSize),
                systemlogService.getQueryWrapper(logQueryRequest));
        List<Systemlog> systemlogList = systemlogPage.getRecords();
        List<BusinessLogVO> businessLogVOList = systemlogList.stream().map(item -> {
            BusinessLogVO businessLogVO = new BusinessLogVO();
            BeanUtils.copyProperties(item, businessLogVO);
            //查询商家信息
            Long userId = item.getUserId();
            Business business = null;
            if (userId != null && userId > -1) {
                QueryWrapper<Business> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("userId", userId);
                business = businessService.getOne(queryWrapper);
            }
            businessLogVO.setBusiness(business);
            return businessLogVO;
        }).collect(Collectors.toList());
        Page<BusinessLogVO> businessLogVOPage = new Page<>(current, pageSize, systemlogPage.getTotal());
        businessLogVOPage.setRecords(businessLogVOList);
        return ResultUtils.success(businessLogVOPage);
    }
}
