package com.bsz.school_send_back_end.listener;

import com.bsz.school_send_back_end.contant.RabbitMQConstant;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.domain.PrivateMessage;
import com.bsz.school_send_back_end.utils.MultiDelayMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;

import jakarta.annotation.Resource;
import java.util.*;

@Component
@Slf4j
public class RabbitMQSender {

    @Resource
    private AmqpTemplate amqpTemplate;

    @Resource
    private RabbitTemplate rabbitTemplate;

    public void sendOrderMessage (Orders message) {
        log.info("消息队列 MQSender 业务消息发送 send OrderMessage:" + message);
        amqpTemplate.convertAndSend(RabbitMQConstant.TOPIC_QUEUE2,message, msg->{
            msg.getMessageProperties().setReplyTo(RabbitMQConstant.ORDER_REPLY_QUEUE);
            return msg;
        });
    }


    public void sendPrivateMessage(PrivateMessage message) {
        log.info("消息队列 MQSender 业务消息发送 send message:" + message);
        amqpTemplate.convertAndSend(RabbitMQConstant.TOPIC_QUEUE1,message, msg->{
            msg.getMessageProperties().setReplyTo(RabbitMQConstant.TOPIC_QUEUE1);
            return msg;
        });
    }

//    public void sendDelayedOrderMessage(long orderId, long delayMillis) {
//        Map<String, Object> headers = new HashMap<>();
//        headers.put("x-delay", delayMillis); // 设置延迟时间（毫秒）
//
//        amqpTemplate.convertAndSend(
//                RabbitMQConstant.DELAYED_EXCHANGE, // 交换机
//                RabbitMQConstant.DELAYED_ROUTING_KEY, // 路由键
//                orderId, // 消息内容（订单 ID）
//                message -> {
//                    message.getMessageProperties().getHeaders().putAll(headers);  // 将延迟时间添加到消息头部
//                    return message;
//                });
//        log.info("发送延迟订单取消消息，订单ID: " + orderId + "，延迟时间: " + delayMillis + "ms");
//    }
//
//
//    public void sendDelayedMessage() {
//        // 设置延迟时间为 5 秒
//        MessageProperties messageProperties = new MessageProperties();
//        messageProperties.setHeader("x-delay", 5000); // 延迟时间（单位：毫秒）
//        Message message = new Message("Hello, delayed world!".getBytes(), messageProperties);
//
//        // 发送到延迟交换机
//        amqpTemplate.send(RabbitMQConstant.DELAYED_EXCHANGE, RabbitMQConstant.DELAYED_ROUTING_KEY, message);
//        log.info("发送延迟成功：" + new Date());
//    }

    public void sendCreateOrderMessage(Long orderId) {
        // 复制 DELAY_MILLIS 到一个新的 ArrayList
        List<Long> newDelayMillis = new ArrayList<>(RabbitMQConstant.DINE_IN_DELAY_MILLIS);
        // 延迟检查订单状态信息
        MultiDelayMessage<Long> msg = new MultiDelayMessage<>(orderId, newDelayMillis);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(
                RabbitMQConstant.CANCEL_ORDER_EXCHANGE,
                RabbitMQConstant.CANCEL_ORDER_ROUTING_KEY,
                msg,
                message -> {
                    // 新版本设置延迟时间的方式
                    message.getMessageProperties()
                            .setHeader("x-delay", delayValue);  // 单位：毫秒
                    return message;
                }
        );
        log.info("发送成功，orderID：{}", orderId);
    }
    //自动退款30分钟未接单的
    public void sendCreateDeliveryOrderMessage (Long orderId) {
        List newDelayMillis = new ArrayList<>(RabbitMQConstant.DELIVERY_DELAY_MILLIS); // 假设延迟时间在此列表中
        MultiDelayMessage msg = new MultiDelayMessage<>(orderId, newDelayMillis);
        int delayValue = msg.removeNextDelay().intValue();

        rabbitTemplate.convertAndSend(
                RabbitMQConstant.DELIVERY_DELAY_EXCHANGE,
                RabbitMQConstant.DELIVERY_ORDER_ROUTING_KEY,
                msg,
                message -> {
                    // 新版使用x-delay头设置延迟
                    message.getMessageProperties()
                            .setHeader("x-delay", delayValue);  // 单位毫秒
                    return message;
                }
        );

        log.info("外卖单延迟消息发送成功，订单ID：{}", orderId);
    }

}