package com.bsz.school_send_back_end.scheduler;

import com.alipay.api.AlipayApiException;
import com.alipay.api.domain.AlipayTradeRefundModel;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.config.AlipayClients;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.service.OrdersService;
import com.bsz.school_send_back_end.utils.UniqueNumberGenerator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import jakarta.annotation.Resource;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Slf4j
@Component
public class CancelOrder {

    @Resource
    private OrdersService ordersService;

    @Scheduled(cron = "0 0/30 * * * *")  // 每30分钟执行一次
    public void cancelOrder() throws AlipayApiException {
        log.info("定时任务开始执行");

        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MINUTE, -30);
        Date halfHourAgo = calendar.getTime();

        // 查询超时订单
        List<Orders> cancelOrders = ordersService.findCancelOrders(halfHourAgo);
        log.info("查询到的超时订单数量：{}", cancelOrders.size());

        if (CollectionUtils.isEmpty(cancelOrders)) {
            log.info("当前没有订单超时");
            return;
        }

        // 遍历每个订单进行处理
        for (Orders orders : cancelOrders) {
            try {
                log.info("正在处理超时订单，订单ID：{}", orders.getId());
                this.closeOrder(orders.getPickupCode()); // 关闭订单或退款
                log.info("订单超时取消成功，订单ID：{}", orders.getId());
            } catch (Exception e) {
                log.error("处理订单超时取消失败，订单ID：{}", orders.getId(), e);
            }
        }

        log.info("所有超时订单处理完成！");
    }

    public void closeOrder(String orderNo) throws AlipayApiException {

        AlipayClients clients = new AlipayClients();
        AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();
        AlipayTradeRefundModel model = new AlipayTradeRefundModel();


        QueryWrapper<Orders> ordersQueryWrapper = new QueryWrapper<>();
        ordersQueryWrapper.eq("pickupCode", orderNo);
        log.info("orderNo：" + orderNo);
        Orders orders = ordersService.getOne(ordersQueryWrapper);
        model.setOutTradeNo(orderNo);
        model.setRefundAmount(String.valueOf(orders.getTotalPrice()));
        ThrowUtils.throwIf(!orders.getState().equals(1) || orders.getState().equals(2),ErrorCode.SYSTEM_ERROR,"当前订单未支付");
        //退款请求单号 要求唯一 需改
        String number = UniqueNumberGenerator.generateNumber();
        model.setOutRequestNo(number);
        log.info("outRequestNo:" + number);
        request.setBizModel(model);
        orders.setState(2);
        boolean update = ordersService.updateById(orders);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "修改订单状态失败");
        log.info("orderState:" + orders.getState());
        AlipayTradeRefundResponse response = clients.alipayClient().execute(request);
        if (!response.isSuccess()) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "退款失败");
        }
        log.info("退款成功");

        log.info(response.getBody());
    }
}
