package com.bsz.school_send_back_end.scheduler;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.*;
import com.bsz.school_send_back_end.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import jakarta.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

@Component
@Slf4j
public class UpdateOrder {

    @Resource
    private OrdersService ordersService;

    @Resource
    private ErrandIncomeService errandIncomeService;

    @Resource
    private ErrandService errandService;

    @Resource
    private ErrandBillService billService;

    @Resource
    private UserService userService;

    @Scheduled(cron = "0 0 * * * ?") // 1小时执行一次
    public void completeExpiredOrders() {
        // 获取当前时间的 24 小时前
        LocalDateTime deadline = LocalDateTime.now().minusHours(24);

        // 查询已出餐且超过 24 小时的订单
        List<Orders> expiredOrders = ordersService.findExpiredOrders(deadline);

        if (CollectionUtils.isEmpty(expiredOrders)) {
            return;
        }

        // 订单 ID 列表
        List<Long> expiredOrderIds = expiredOrders.stream()
                .filter(order -> order.getLocation() != null && !order.getLocation().isEmpty())
                .map(Orders::getId)
                .collect(Collectors.toList());
        // **如果存在需要结算的订单**
        if (!CollectionUtils.isEmpty(expiredOrderIds)) {
            // 查询 `ErrandIncome` 中相关的收入记录
            LambdaQueryWrapper<ErrandIncome> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(ErrandIncome::getOrderId, expiredOrderIds);
            List<ErrandIncome> errandIncomes = errandIncomeService.list(queryWrapper);

            // 更新收入状态
            errandIncomes.forEach(income -> {
                income.setState(1);
                Errand errand = errandService.getById(income.getErrandId());
                Orders orders = ordersService.getById(income.getOrderId());
                User user = userService.getById(orders.getUserId());
                if (errand != null) {
                    BigDecimal totalPrice = errand.getTotalPrice();
                    totalPrice = totalPrice.add(income.getIncome());
                    errand.setTotalPrice(totalPrice);

                    boolean update = errandService.updateById(errand);
                    ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "收入加入总金额失败");

                    ErrandBill errandBill = new ErrandBill();
                    errandBill.setOrderId(orders.getId());
                    errandBill.setErrandState(income.getState());
                    errandBill.setIncome(income.getIncome());
                    errandBill.setUsername(user.getUsername());
                    errandBill.setOrderStartTime(orders.getCreateTime());
                    boolean save = billService.save(errandBill);
                    ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "添加跑腿账单失败");
                }
            });  // 1 = 已结算 将收入加入总金额
            boolean updatedIncome = errandIncomeService.updateBatchById(errandIncomes);
            ThrowUtils.throwIf(!updatedIncome, ErrorCode.OPERATION_ERROR, "修改订单收入状态失败");
        }

        // 更新订单状态
        for (Orders order : expiredOrders) {
            order.setState(5);
        }

        boolean updated = ordersService.updateBatchById(expiredOrders);
        if (updated) {
            log.info("成功完成 {} 个超时订单", expiredOrders.size());
        } else {
            log.warn("自动完成订单时发生错误");
        }
    }
}
