package com.bsz.school_send_back_end.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.bsz.school_send_back_end.model.domain.OrderDetails;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.dto.order.OrderQueryRequest;
import com.bsz.school_send_back_end.model.dto.order.OrderUpdateRequest;
import com.bsz.school_send_back_end.model.vo.OrderErrandVO;
import com.bsz.school_send_back_end.model.vo.OrderStatusVO;
import com.bsz.school_send_back_end.model.vo.OrdersPickVO;
import com.bsz.school_send_back_end.model.vo.OrdersVO;


import jakarta.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

/**
 * @author xy
 */
public interface OrdersService extends IService<Orders> {

    /**
     * 创建订单
     *
     * @return 订单id
     */
    long addOrder(Orders orders, List<OrderDetails> orderDetailsList);

    /**
     * 校验订单
     */
    void validOrder(Orders orders);

    /**
     * 获取订单视图信息
     */
    OrdersVO getOrdersVO(Orders orders);

    List<OrdersVO> getOrdersVO(List<Orders> ordersList);

    /**
     * 获取查询条件
     */
    QueryWrapper<Orders> getQueryWrapper(OrderQueryRequest orderQueryRequest);

    /**
     * 悲观锁
     */
    Orders getOrderWithPessimisticLock(Long ordersId);

    /**
     * 修改未支付订单内容
     */
    Boolean OrderToUpdate(OrderUpdateRequest orderUpdateRequest, HttpServletRequest request);

    /**
     * 修改超时订单
     */
    List<Orders> findExpiredOrders(LocalDateTime deadline);


    /**
     * 查询超过半个小时无人抢单的订单
     */
    List<Orders> findCancelOrders(Date halfHourAgo);

    /**
     * 跑腿查看订单详情
     */
    OrderErrandVO getOrderErrandVO(Orders orders);

    /**
     * 跑腿查看订单详情列表
     */
    List<OrderErrandVO> getListOrderErrandVO(List<Orders> list);

    /**
     * 获取订单状态列表
     */
    List<OrderStatusVO> liststatus(HttpServletRequest request);

    /**
     * 获取七天内的订单数量
     */
    List<OrdersPickVO> getOrdernumber();

}
