package com.bsz.school_send_back_end.service;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.bsz.school_send_back_end.model.vo.LoginUserVO;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.user.UserSearchRequest;
import com.bsz.school_send_back_end.model.dto.user.UserUpdatePasswordRequest;
import org.springframework.stereotype.Service;

import jakarta.servlet.http.HttpServletRequest;

/**
* @author xy
*/
@Service
public interface UserService extends IService<User> {

    /**
     * 用户注册
     *
     * @param userAccount 登陆账号
     * @param userPassword 登录密码
     * @param checkPassword 校验密码
     * @return 新用户id
     */
    long userRegister(String userAccount, String userPassword, String checkPassword);

    /**
     *
     * @param userAccount 登陆账号
     * @param userPassword 登陆密码
     * @param request 返回请求
     * @return 脱敏后的用户信息
     */

    String userLogin(String userAccount, String userPassword, String appName, HttpServletRequest request);


    /**
     * 用户脱敏
     *
     * @param originUser
     * @return
     */
    User getSafetyUser(User originUser);

    /**
     * 用户注销
     *
     *
     * @param request
     * @return 1 表示成功
     */
    int userLogout(HttpServletRequest request);

    /**
     * 获取当前用户登录信息
     *
     * @param request
     * @return
     */
    User getLoginUser(HttpServletRequest request);

    /**
     * 修改密码
     *
     * @param updatePasswordRequest
     * @param request
     */
    boolean updateUserPassword(UserUpdatePasswordRequest updatePasswordRequest, HttpServletRequest request);

    /**
     * 分页条件
     * @param searchRequest
     * @return
     */
    QueryWrapper<User> getQueryWrapper(UserSearchRequest searchRequest);

    /**
     * 用户登录(支付宝)
     * @param openId
     * @param userInfo
     * @param request
     * @return 返回脱敏后的用户信息
     */
    LoginUserVO userLoginByAlipay(String openId, JSONObject userInfo, HttpServletRequest request);

    public LoginUserVO getLoginUserVO(User user);

    /**
     *校验
     */
    void validUser(User user, boolean add);
}
