package com.bsz.school_send_back_end.service.impl;


import com.alipay.api.AlipayApiException;
import com.alipay.api.domain.AlipayTradeCreateModel;
import com.alipay.api.request.AlipayTradeCreateRequest;
import com.alipay.api.response.AlipayTradeCreateResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.config.AlipayClients;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.OrderDetails;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.service.AliPayService;
import com.bsz.school_send_back_end.service.OrderDetailsService;
import com.bsz.school_send_back_end.service.OrdersService;
import com.bsz.school_send_back_end.utils.UniqueNumberGenerator;
import lombok.extern.slf4j.Slf4j;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import jakarta.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AlipayServiceImpl implements AliPayService {

    @Resource
    private OrdersService ordersService;

    @Resource
    private OrderDetailsService orderDetailsService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized String createPayment(String orderId, String miniOpenId, BigDecimal amount) {
        //创建客户
        AlipayClients clients = new AlipayClients();
        //AlipayClient client = clients.alipayClient();
        //创建交易请求
        AlipayTradeCreateRequest aliRequest = new AlipayTradeCreateRequest();
        // 构造 JSON 字符串作为 biz_content
        JSONObject bizContent = new JSONObject();
        //请求参数 沙箱内部参数

        String outTradeNo = UniqueNumberGenerator.generateNumber();

        Orders order = ordersService.getById(orderId);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + outTradeNo);
        }

        LambdaQueryWrapper<OrderDetails> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(OrderDetails::getOrderId, orderId);
        List<OrderDetails> orderDetailsList = orderDetailsService.list(wrapper);
        String subject = orderDetailsList.stream()
                .map(OrderDetails::getAttributeNames)
                .collect(Collectors.joining(", "));
        //给支付宝的订单号设置成取餐码
        order.setPickupCode(outTradeNo);
        boolean update = ordersService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "修改订单状态失败");

        try {
            bizContent.put("out_trade_no", outTradeNo);
            bizContent.put("total_amount", amount.toString());
            bizContent.put("subject", subject);
            bizContent.put("product_code", "JSAPI_PAY");
            bizContent.put("seller_id", "2088932829893612");
            bizContent.put("buyer_open_id", miniOpenId); // ✅ 关键字段，放进 biz_content 中
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
        ///回调地址
        aliRequest.setBizContent(bizContent.toString());
        aliRequest.setNotifyUrl("https://xiaokuaisong.shop:6448/api/Alipay/notifyUrl");
        String trade_no = null;
        try {
            AlipayTradeCreateResponse response = clients.alipayClient().execute(aliRequest);
            trade_no = response.getTradeNo();
            if (trade_no == null) {
                throw new BusinessException(ErrorCode.NULL_ERROR, "未获取到trade_no");
            }
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        log.info("trade_no:" + trade_no);
        return trade_no;
    }
}
