package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.mapper.AttributeMapper;
import com.bsz.school_send_back_end.model.domain.Attribute;
import com.bsz.school_send_back_end.model.dto.attribute.AttributeAddRequest;
import com.bsz.school_send_back_end.model.dto.attribute.AttributeUpdateRequest;
import com.bsz.school_send_back_end.model.vo.AttributeVO;
import com.bsz.school_send_back_end.model.vo.SpecificationsVO;
import com.bsz.school_send_back_end.service.AttributeService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author xy
*/
@Service
public class AttributeServiceImpl extends ServiceImpl<AttributeMapper, Attribute>
    implements AttributeService {

    @Override
    public List<SpecificationsVO> getBySpecificationsVoId(List<SpecificationsVO> specificationsVOList) {
        if (CollectionUtils.isEmpty(specificationsVOList)) {
            return specificationsVOList;
        }
        return specificationsVOList.stream().peek(item ->{
            Long specificationsId = item.getId();
            //根据规格id获取属性
            QueryWrapper<Attribute> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("specificationsId",specificationsId);
            List<Attribute> attributeList = this.list(queryWrapper);
            //获取属性脱敏信息
            List<AttributeVO> attributeVOList = getAttributeVOList(attributeList);
            item.setAttributeVOList(attributeVOList);
        }).collect(Collectors.toList());
    }

    @Override
    public Boolean addAttribute(List<AttributeAddRequest> attributeAddRequests, Long businessId, Long specificationsId) {
        if (attributeAddRequests.isEmpty()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        //保存每个规格属性
        for (AttributeAddRequest attributeAddRequest: attributeAddRequests) {
            Attribute attribute = new Attribute();
            BeanUtils.copyProperties(attributeAddRequest, attribute);
            //校验
            validAttribute(attribute);
            //添加保存
            saveAttribute(businessId, specificationsId, attribute);
        }

        return true;
    }

    @Override
    public void validAttribute(Attribute attribute) {
        if  (attribute == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        String attributeName = attribute.getAttributeName();
        Integer attributeStatus = attribute.getAttributeStatus();

        if (attributeStatus == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请填写属性状态");
        }

        if (StringUtils.isBlank(attributeName)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请填写属性名称");
        }
    }

    @Override
    public Boolean updateAttribute(List<AttributeUpdateRequest> attributeUpdateRequests, Long businessId, Long specificationsId) {
        if (attributeUpdateRequests.isEmpty()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"请补全属性参数");
        }
        QueryWrapper<Attribute> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("specificationsId",specificationsId);
        boolean remove = this.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);
        //更新
        for (AttributeUpdateRequest attributeUpdateRequest : attributeUpdateRequests) {
            Attribute attribute = new Attribute();
            BeanUtils.copyProperties(attributeUpdateRequest,attribute);
            //校验
            validAttribute(attribute);
            //更新
            saveAttribute(businessId,specificationsId,attribute);
        }

        return true;
    }

    /**
     * 获取脱敏属性列表
     * @param attributes 为脱敏列表
     * @return 脱敏列表
     */
    private List<AttributeVO> getAttributeVOList (List<Attribute> attributes) {
        if (CollectionUtils.isEmpty(attributes)) {
            return new ArrayList<>();
        }
            return attributes.stream().map(item -> {
                AttributeVO attributeVO = new AttributeVO();
                BeanUtils.copyProperties(item,attributeVO);
                return attributeVO;
            }).collect(Collectors.toList());
    }

    /**
     * 保存属性信息
     */
    private void saveAttribute(Long businessId, Long specificationsId, Attribute attribute) {
        attribute.setBusinessId(businessId);
        attribute.setSpecificationsId(specificationsId);
        boolean result = this.save(attribute);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
    }
}




