package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Collect;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.collect.CollectAddRequest;
import com.bsz.school_send_back_end.model.vo.CollectVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.CollectService;
import com.bsz.school_send_back_end.mapper.CollectMapper;
import com.bsz.school_send_back_end.service.UserService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author xy
*/
@Service
public class CollectServiceImpl extends ServiceImpl<CollectMapper, Collect>
    implements CollectService{

    @Resource
    private UserService userService;

    @Resource
    private BusinessService businessService;
    @Override
    public void validCollect(CollectAddRequest collectAddRequest) {
        Long businessId = collectAddRequest.getBusinessId();
        ThrowUtils.throwIf(businessId == null, ErrorCode.PARAMS_ERROR, "参数不全");
        LambdaQueryWrapper<Business> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Business::getId, businessId);
        long count = businessService.count(wrapper);
        ThrowUtils.throwIf(count != 1, ErrorCode.PARAMS_ERROR, "填写的商家id不存在");
    }

    @Override
    public Boolean addCollect(Collect collect, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long loginUserId = loginUser.getId();
        Long businessId = collect.getBusinessId();
        LambdaQueryWrapper<Collect> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Collect::getBusinessId, businessId);
        wrapper.eq(Collect::getUserId, loginUserId);
        long count = this.count(wrapper);
        ThrowUtils.throwIf(count > 0, ErrorCode.OPERATION_ERROR, "该商家已收藏");
        collect.setUserId(loginUserId);
        return this.save(collect);
    }

    @Override
    public List<CollectVO> getCollectList(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long loginUserId = loginUser.getId();
        LambdaQueryWrapper<Collect> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Collect::getUserId, loginUserId);
        List<Collect> collectList = this.list(wrapper);

        return collectList.stream().map(item -> {
            CollectVO collectVO = new CollectVO();
            BeanUtils.copyProperties(item, collectVO);
            Business business = businessService.getOne(Wrappers.<Business>lambdaQuery()
                    .eq(Business::getId, item.getBusinessId()));
            collectVO.setBusiness(business);
            return collectVO;
        }).collect(Collectors.toList());
    }
}




