package com.bsz.school_send_back_end.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.mapper.DishesGroupMapper;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.DishesGroup;
import com.bsz.school_send_back_end.model.vo.DishesGroupVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.DishesGroupService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author xy
*
*/
@Service
public class DishesGroupServiceImpl extends ServiceImpl<DishesGroupMapper, DishesGroup>
    implements DishesGroupService {

    @Resource
    private BusinessService businessService;

    @Override
    public void validDishesGroup(DishesGroup dishesGroup, boolean update) {
        if (dishesGroup == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String groupName = dishesGroup.getGroupName();
        Integer isTopping = dishesGroup.getIsTopping();
        Long dishesGroupId = dishesGroup.getId();

        if (update) {
            if (dishesGroupId == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR,"未填写分组id");
            }
        }

        if (isTopping == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"请选择是否置顶");
        }

        if (StringUtils.isBlank(groupName)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"未填写分组名称");
        }
    }

    @Override
    public Boolean addDishesGroup(DishesGroup dishesGroup, HttpServletRequest request) {
        String groupName = dishesGroup.getGroupName();
        Integer isTopping = dishesGroup.getIsTopping();

        //获取商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        dishesGroup.setBusinessId(businessId);

        //判断更新的分组是否置顶，并将组内数据更改为非置顶
        changeTopping(businessId,isTopping);
        QueryWrapper<DishesGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("businessId",businessId);
        queryWrapper.eq("groupName",groupName);
        long count = this.count(queryWrapper);
        if (count > 0) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR,"存在相同的分组");
        }

        return this.save(dishesGroup);
    }



    @Override
    public List<DishesGroupVO> getDishGroupVOList(List<DishesGroup> dishesGroupList) {
        if (CollectionUtils.isEmpty(dishesGroupList)) {
            return new ArrayList<>();
        }

        return dishesGroupList.stream().map(item -> {
            DishesGroupVO dishesGroupVO = new DishesGroupVO();
            BeanUtils.copyProperties(item, dishesGroupVO);
            return dishesGroupVO;
        }).collect(Collectors.toList());
    }

    @Override
    public Boolean updateDishesGroupById(DishesGroup dishesGroup, HttpServletRequest request) {
        //获取当前商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        Long dishesGroupId = dishesGroup.getId();
        String groupName = dishesGroup.getGroupName();
        Integer isTopping = dishesGroup.getIsTopping();
        //判断更新的分组是否置顶，并将组内数据更改为非置顶
        changeTopping(businessId,isTopping);
        //更新分组信息
        UpdateWrapper<DishesGroup> wrapper = new UpdateWrapper<>();
        wrapper.eq("id",dishesGroupId);
        wrapper.eq("businessId",businessId);
        wrapper.set("groupName",groupName);
        wrapper.set("isTopping",isTopping);
        return this.update(wrapper);
    }


    private void changeTopping(Long businessId, Integer isTopping) {
        QueryWrapper<DishesGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("businessId",businessId);
        List<DishesGroup> list = this.list();

        if (isTopping == 1 && list.size() > 0) {
            List<DishesGroup> collect = list.stream().peek(item -> {
                item.setIsTopping(0);
            }).collect(Collectors.toList());
            boolean updateTopping = this.updateBatchById(collect);
            ThrowUtils.throwIf(!updateTopping,ErrorCode.OPERATION_ERROR);
        }
    }
}




