package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.mapper.DishesMapper;
import com.bsz.school_send_back_end.model.domain.Dishes;
import com.bsz.school_send_back_end.model.domain.SpecificationsDishes;
import com.bsz.school_send_back_end.model.dto.Dishes.DishesQueryRequest;
import com.bsz.school_send_back_end.model.vo.DishesVO;
import com.bsz.school_send_back_end.service.DishesService;
import com.bsz.school_send_back_end.service.SpecificationsDishesService;
import com.bsz.school_send_back_end.utils.SqlUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import jakarta.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author xy
*/
@Service
public class DishesServiceImpl extends ServiceImpl<DishesMapper, Dishes>
    implements DishesService {

    @Resource
    private SpecificationsDishesService specificationsDishesService;

    @Override
    public void removeByDishesGroupId(Long dishesGroupId) {
        QueryWrapper<Dishes> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("dishesGroupId",dishesGroupId);
        List<Dishes> list = this.list(queryWrapper);
        //判断该分组是否有菜品
        if (list.isEmpty()) {
            return;
        }
        boolean remove = this.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);
        //删除中间表数据
        for (Dishes dishes : list) {
            Long dishesId = dishes.getId();
            specificationsDishesService.deleteByDishesId(dishesId);
        }
    }

    @Override
    public void validDishes(Dishes dishes, Boolean update) {
        Long id = dishes.getId();
        Long dishesGroupId = dishes.getDishesGroupId();
        String dishesName = dishes.getDishesName();
        Double dishesPrice = dishes.getDishesPrice();
        String status = dishes.getStatus();
        Integer inventoryStatus = dishes.getInventoryStatus();

        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR);
            }
        }

        if (StringUtils.isAnyBlank(dishesName,status)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        if (dishesGroupId == null || dishesPrice == null || inventoryStatus == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
    }

    @Override
    public Long addDishes(Dishes dishes) {
        //判断菜品是否存在
        validDishesName(dishes);
        //保存代码
        boolean save = this.save(dishes);
        ThrowUtils.throwIf(!save,ErrorCode.OPERATION_ERROR);
        return dishes.getId();
    }

    @Override
    public Boolean updateDishes(Dishes dishes) {
        //判断菜品是否存在
        validDishesName(dishes);
        //更新菜品
        return this.updateById(dishes);
    }

    @Override
    public List<DishesVO> getDishesVOList(List<Dishes> dishesList) {
        if (CollectionUtils.isEmpty(dishesList)) {
            return new ArrayList<>();
        }
        return dishesList.stream().map(item->{
            long count = getCount(item);
            DishesVO dishesVO = new DishesVO();
            BeanUtils.copyProperties(item,dishesVO);
            if (count > 0) {
                dishesVO.setIsSpecification(1);
            }
            return dishesVO;
        }).collect(Collectors.toList());
    }

    @Override
    public DishesVO getDishesVO(Dishes dishes) {
        if (dishes == null) {
            return null;
        }
        long count = getCount(dishes);
        DishesVO dishesVO = new DishesVO();
        BeanUtils.copyProperties(dishes, dishesVO);
        if (count > 0) {
            dishesVO.setIsSpecification(1);
        }
        return dishesVO;
    }

    @Override
    public QueryWrapper<Dishes> getWrapper(DishesQueryRequest dishesQueryRequest) {
        Long businessId = dishesQueryRequest.getBusinessId();
        Long dishesGroupId = dishesQueryRequest.getDishesGroupId();
        String status = dishesQueryRequest.getStatus();
        String dishesName = dishesQueryRequest.getDishesName();
        String sortField = dishesQueryRequest.getSortField();
        String sortOrder = dishesQueryRequest.getSortOrder();
        QueryWrapper<Dishes> wrapper = new QueryWrapper<>();
        wrapper.eq(businessId != null, "businessId", businessId);
        wrapper.eq(dishesGroupId != null, "dishesGroupId", dishesGroupId);
        wrapper.eq(StringUtils.isNotBlank(status), "status", status);
        wrapper.like(StringUtils.isNotBlank(dishesName), "dishesName", dishesName);
        wrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC), sortField);
        return wrapper;
    }

    private void validDishesName(Dishes dishes) {
        QueryWrapper<Dishes> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("dishesName",dishes.getDishesName());
        queryWrapper.eq("businessId",dishes.getBusinessId());
        long count = this.count(queryWrapper);
        if (dishes.getId() == null) {
            ThrowUtils.throwIf(count > 0,ErrorCode.PARAMS_ERROR,"已存在该菜品");
        }
        else {
            ThrowUtils.throwIf(count > 1,ErrorCode.PARAMS_ERROR,"已存在该菜品");
        }
    }

    private long getCount(Dishes dishes) {
        Long dishesId = dishes.getId();
        QueryWrapper<SpecificationsDishes> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("dishesId",dishesId);
        return specificationsDishesService.count(queryWrapper);
    }
}




