package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.ErrandAuth;
import com.bsz.school_send_back_end.model.dto.errandAtuh.ErrandAuthQueryRequest;
import com.bsz.school_send_back_end.service.ErrandAuthService;
import com.bsz.school_send_back_end.mapper.ErrandAuthMapper;
import com.bsz.school_send_back_end.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

/**
* @author xy
*/
@Service
public class ErrandAuthServiceImpl extends ServiceImpl<ErrandAuthMapper, ErrandAuth>
    implements ErrandAuthService{

    @Override
    public void validErrandAuth(ErrandAuth errandAuth, boolean add) {
        Long errandId = errandAuth.getErrandId();
        String bankCard = errandAuth.getBankCard();
        String frontIdCard = errandAuth.getFrontIdCard();
        String backIdCard = errandAuth.getBackIdCard();
        if (add) {
            if (StringUtils.isAnyBlank(backIdCard,frontIdCard,bankCard)) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "信息不全");
            }
        }
        else {
            LambdaQueryWrapper<ErrandAuth> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(ErrandAuth::getErrandId, errandId);
            long count = this.count(queryWrapper);
            ThrowUtils.throwIf(count == 0, ErrorCode.PARAMS_ERROR, "该用户未关联跑腿");
        }
    }

    @Override
    public LambdaUpdateWrapper<ErrandAuth> getUpdateWrapper(ErrandAuth errandAuth) {
        Long errandId = errandAuth.getErrandId();
        String bankCard = errandAuth.getBankCard();
        String backIdCard = errandAuth.getBackIdCard();
        String frontIdCard = errandAuth.getFrontIdCard();
        LambdaUpdateWrapper<ErrandAuth> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(StringUtils.isNotBlank(bankCard), ErrandAuth::getBankCard, bankCard);
        updateWrapper.set(StringUtils.isNotBlank(backIdCard), ErrandAuth::getBackIdCard, backIdCard);
        updateWrapper.set(StringUtils.isNotBlank(frontIdCard), ErrandAuth::getFrontIdCard, frontIdCard);
        updateWrapper.eq(ErrandAuth::getErrandId, errandId);
        return updateWrapper;
    }

    @Override
    public QueryWrapper<ErrandAuth> getQueryWrapper(ErrandAuthQueryRequest errandAuthQueryRequest) {
        Long id = errandAuthQueryRequest.getId();
        Long errandId = errandAuthQueryRequest.getErrandId();
        String sortField = errandAuthQueryRequest.getSortField();
        String sortOrder = errandAuthQueryRequest.getSortOrder();
        QueryWrapper<ErrandAuth> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        wrapper.eq(ObjectUtils.isNotEmpty(errandId), "errandId", errandId);
        wrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);

        return wrapper;
    }

    @Override
    public ErrandAuth getErrandAuth(Long errandId) {
        LambdaQueryWrapper<ErrandAuth> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ErrandAuth::getErrandId, errandId);
        ErrandAuth errandAuth = this.getOne(queryWrapper);
        ThrowUtils.throwIf(errandAuth == null, ErrorCode.PARAMS_ERROR, "当前跑腿id没有认证");

        return errandAuth;
    }
}




