package com.bsz.school_send_back_end.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.ErrandIncome;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.dto.erradnIncome.ErrandIncomeQueryRequest;
import com.bsz.school_send_back_end.service.ErrandIncomeService;
import com.bsz.school_send_back_end.mapper.ErrandIncomeMapper;
import com.bsz.school_send_back_end.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

/**
* @author xy
*/
@Service
public class ErrandIncomeServiceImpl extends ServiceImpl<ErrandIncomeMapper, ErrandIncome>
    implements ErrandIncomeService{

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addErrandIncome(Orders orders) {
        String location = orders.getLocation();
        Long errandId = orders.getErrandId();
        Long ordersId = orders.getId();
        ErrandIncome errandIncome = new ErrandIncome();
        errandIncome.setErrandId(errandId);
        errandIncome.setOrderId(ordersId);
        errandIncome.setIncome(UserConstant.DELIVERY_FEE_MAP.get(location));
        boolean save = this.save(errandIncome);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "添加配送费失败");
    }


    @Override
    public QueryWrapper<ErrandIncome> getWrapper(ErrandIncomeQueryRequest queryRequest) {
        Long id = queryRequest.getId();
        Long orderId = queryRequest.getOrderId();
        Long errandId = queryRequest.getErrandId();
        Integer state = queryRequest.getState();
        String sortField = queryRequest.getSortField();
        String sortOrder = queryRequest.getSortOrder();
        QueryWrapper<ErrandIncome> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotEmpty(id),"id", id);
        wrapper.eq(ObjectUtils.isNotEmpty(orderId), "orderId", orderId);
        wrapper.eq(ObjectUtils.isNotEmpty(errandId), "errandId",errandId);
        wrapper.eq(ObjectUtils.isNotEmpty(state), "state", state);
        wrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);

        return wrapper;
    }

    @Override
    public LambdaQueryWrapper<ErrandIncome> getQueryWrapper(Long errandId) {

        // 获取当前日期的开始和结束时间
        LocalDateTime startOfDay = LocalDate.now().atStartOfDay();
        LocalDateTime endOfDay = LocalDate.now().atTime(23, 59, 59, 999999999);

        // 转换为 Date 对象
        Date startDate = Date.from(startOfDay.atZone(ZoneId.systemDefault()).toInstant());
        Date endDate = Date.from(endOfDay.atZone(ZoneId.systemDefault()).toInstant());

        LambdaQueryWrapper<ErrandIncome> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ErrandIncome::getErrandId, errandId);
        queryWrapper.ge(ErrandIncome::getCreateTime, startDate); // 大于等于当天的 00:00:00
        queryWrapper.le(ErrandIncome::getCreateTime, endDate);   // 小于等于当天的 23:59:59
        return queryWrapper;
    }
}




