package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.mapper.UserRatingMapper;
import com.bsz.school_send_back_end.model.domain.ErrandLevel;
import com.bsz.school_send_back_end.service.ErrandLevelService;
import com.bsz.school_send_back_end.mapper.ErrandLevelMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import jakarta.annotation.Resource;
import java.math.BigDecimal;

/**
* @author xy
*/
@Service
@Slf4j
public class ErrandLevelServiceImpl extends ServiceImpl<ErrandLevelMapper, ErrandLevel>
    implements ErrandLevelService{
    
    @Resource
    private ErrandLevelMapper errandLevelMapper;

    @Resource
    private UserRatingMapper userRatingMapper;

    @Override
    public void updateErrandLevel(Long errandId) {
        Double avg = userRatingMapper.findAvgByRatedEntityId(errandId);
        if (avg == null) {
            avg = UserConstant.DEFAULT_AVG_SCORE;
            log.info("warn: avg没有值，将赋初始值1：" + avg);
            log.info("当前跑腿id：" + errandId);
        }
        int level = calculateLevel(avg);
        ErrandLevel errandLevel = errandLevelMapper.selectOne(Wrappers.<ErrandLevel>lambdaQuery()
                .eq(ErrandLevel::getErrandId, errandId));

        if (errandLevel == null) {
            errandLevel = new ErrandLevel();
            errandLevel.setErrandId(errandId);
            errandLevel.setLevel(UserConstant.DEFAULT_LEVEL);
            errandLevel.setAverageScore(BigDecimal.valueOf(1));

        }
        errandLevel.setAverageScore(BigDecimal.valueOf(avg));
        errandLevel.setLevel(level);

        this.saveOrUpdate(errandLevel);
    }

    private int calculateLevel(Double averageScore) {
        if (averageScore >= 4.5) {
            return 5;
        } else if (averageScore >= 4.0) {
            return 4;
        } else if (averageScore >= 3.0) {
            return 3;
        } else if (averageScore >= 2.0) {
            return 2;
        } else {
            return 1;
        }
    }
}




