package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.*;
import com.bsz.school_send_back_end.model.dto.errand.ErrandQueryRequest;
import com.bsz.school_send_back_end.model.dto.errand.ErrandStateRequest;
import com.bsz.school_send_back_end.model.dto.errand.ErrandUpdateRequest;
import com.bsz.school_send_back_end.model.vo.ErrandVO;
import com.bsz.school_send_back_end.service.*;
import com.bsz.school_send_back_end.mapper.ErrandMapper;
import com.bsz.school_send_back_end.utils.RegexUtils;
import com.bsz.school_send_back_end.utils.SqlUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.Objects;

/**
* @author xy
*/
@Service
@Slf4j
public class ErrandServiceImpl extends ServiceImpl<ErrandMapper, Errand>
    implements ErrandService{

    @Resource
    private UserService userService;

    @Resource
    private ErrandAuthService errandAuthService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private ErrandIncomeService errandIncomeService;

    @Resource
    private WebsocketService websocketService;

    @Resource
    private ErrandLevelService errandLevelService;

    @Resource
    private ErrandOrderService errandOrderService;

    @Resource
    private OrderImageService orderImageService;

    @Override
    public void validErrand(Errand errand, boolean add) {
        String errandName = errand.getErrandName();
        Integer gender = errand.getGender();
        String distributionScope = errand.getDistributionScope();
        Long userId = errand.getUserId();
        String phone = errand.getErrandPhone();
        if (add) {
            if (StringUtils.isAnyBlank(errandName, phone) || ObjectUtils.isEmpty(gender) || ObjectUtils.isEmpty(distributionScope)) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "信息不全");
            }
        }
        //校验手机号
        if (StringUtils.isNotBlank(phone)) {
            boolean phoneInvalid = RegexUtils.isPhoneInvalid(phone);
            ThrowUtils.throwIf(phoneInvalid, ErrorCode.PARAMS_ERROR, "手机格式不正确");
        }
        //校验跑腿名
        if (StringUtils.isNotBlank(errandName)) {
            LambdaQueryWrapper<Errand> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(Errand::getUserId, userId);
            long count = this.count(queryWrapper);
            ThrowUtils.throwIf(count > 0, ErrorCode.PARAMS_ERROR, "跑腿名字重复");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized void addErrand(User user, Errand errand) {
        boolean save = userService.save(user);
        ThrowUtils.throwIf(!save, ErrorCode.SYSTEM_ERROR,"用户添加失败");
        errand.setUserId(user.getId());
        boolean save1 = this.save(errand);
        ThrowUtils.throwIf(!save1, ErrorCode.SYSTEM_ERROR,"跑腿添加失败");

        errandLevelService.updateErrandLevel(errand.getId());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized void deleteErrand(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        LambdaQueryWrapper<Errand> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Errand::getUserId, userId);
        Errand errand = this.getOne(queryWrapper);
        ThrowUtils.throwIf(errand == null, ErrorCode.NOT_FOUND_ERROR, "该用户没有关联跑腿");
        Long errandId = errand.getId();
        LambdaQueryWrapper<ErrandAuth> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ErrandAuth::getErrandId, errandId);
        ErrandAuth errandAuth = errandAuthService.getOne(wrapper);
        if (errandAuth != null){
            boolean remove2 = errandAuthService.removeById(errandAuth.getId());
            ThrowUtils.throwIf(!remove2, ErrorCode.SYSTEM_ERROR, "删除跑腿认证失败");
        }
        boolean remove = userService.removeById(userId);
        boolean remove1 = this.removeById(errandId);
        ThrowUtils.throwIf(!remove || !remove1, ErrorCode.SYSTEM_ERROR, "删除用户或跑腿失败");
    }

    @Override
    public Errand getLoginErrand(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        LambdaQueryWrapper<Errand> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Errand::getUserId, userId);
        Errand errand = this.getOne(queryWrapper);
        ThrowUtils.throwIf(errand == null, ErrorCode.NOT_FOUND_ERROR, "当前用户没有关联跑腿");

        return errand;
    }

    @Override
    public QueryWrapper<Errand> getQueryWrapper(ErrandQueryRequest queryRequest) {
        Long id = queryRequest.getId();
        String errandName = queryRequest.getErrandName();
        Integer gender = queryRequest.getGender();
        String errandPhone = queryRequest.getErrandPhone();
        String distributionScope = queryRequest.getDistributionScope();
        Integer state = queryRequest.getState();
        String sortOrder = queryRequest.getSortOrder();
        String sortField = queryRequest.getSortField();
        QueryWrapper<Errand> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id),"id", id);
        queryWrapper.eq(StringUtils.isNotBlank(errandName), "errandName",errandName);
        queryWrapper.eq(ObjectUtils.isNotEmpty(gender), "gender", gender);
        queryWrapper.eq(StringUtils.isNotBlank(errandPhone), "errandPhone", errandPhone);
        queryWrapper.eq(StringUtils.isNotBlank(distributionScope), "distributionScope", distributionScope);
        queryWrapper.eq(ObjectUtils.isNotEmpty(state), "state", state);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField),sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);

        return queryWrapper;
    }

    @Override
    public void orderToErrand(ErrandStateRequest errandStateRequest, HttpServletRequest request) {
        Long ordersId = errandStateRequest.getOrderId();
        Integer errandsState = errandStateRequest.getErrandsState();
        String imageAddress = errandStateRequest.getImageAddress();
        Errand loginErrand = this.getLoginErrand(request);
        Long loginErrandId = loginErrand.getId();
        Orders orders = ordersService.getOrderWithPessimisticLock(ordersId);
        ThrowUtils.throwIf(orders == null, ErrorCode.PARAMS_ERROR, "订单不存在");
        Long errandId = orders.getErrandId();
        ThrowUtils.throwIf(!Objects.equals(loginErrandId, errandId), ErrorCode.PARAMS_ERROR, "当前订单不属于该跑腿");

        ErrandOrder errandOrder = errandOrderService.getOne(Wrappers.<ErrandOrder>lambdaQuery()
                .eq(ErrandOrder::getOrderId, ordersId));

        if (errandsState == 5) {
            orders.setErrandId(null);
            errandOrder.setErrandState(errandsState);//再次抢单
            errandOrder.setErrandId(null);
            boolean update = ordersService.updateById(orders);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "取消订单失败");
            log.info("跑腿取消接单，订单ID: {}", ordersId);
            websocketService.sendOrderToErrand(orders);
        }else if (errandsState == 3) {
            ThrowUtils.throwIf(orders.getState() != 4, ErrorCode.OPERATION_ERROR, "当前商家未出餐"); //商家未出餐，不允许取货
            errandOrder.setErrandState(errandsState);// 已取餐
            boolean update = ordersService.updateById(orders);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "取餐失败");
            log.info("订单已取餐，订单id：" + ordersId);
        } else if (errandsState == 4) {
            ThrowUtils.throwIf(errandOrder.getErrandState() != 3, ErrorCode.OPERATION_ERROR, "商家未取货");
            errandOrder.setErrandState(errandsState);// 订单已送达
            boolean update = ordersService.updateById(orders);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单送达失败");

            if (imageAddress == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "请先上传图片");
            }
            OrderImage orderImage = new OrderImage();
            orderImage.setOrderId(ordersId);
            orderImage.setImageAddress(imageAddress);
            orderImageService.save(orderImage);
            log.info("订单已送达，订单id：" + ordersId);
            errandIncomeService.addErrandIncome(orders);
            this.updateTotal(loginErrand, false);
        }

        boolean update = errandOrderService.updateById(errandOrder);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "更新跑腿关联失败");
    }

    @Override
    public Boolean updateErrand(ErrandUpdateRequest updateRequest) {
        Long id = updateRequest.getId();
        String errandName = updateRequest.getErrandName();
        String errandPhone = updateRequest.getErrandPhone();
        String distributionScope = updateRequest.getDistributionScope();
        Integer gender = updateRequest.getGender();
        Integer state = updateRequest.getState();

        Errand errand = this.getOne(Wrappers.<Errand>lambdaQuery().eq(Errand::getId, id));
        ThrowUtils.throwIf(errand == null, ErrorCode.PARAMS_ERROR, "当前跑腿id不正确");

        LambdaUpdateWrapper<Errand> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(Errand::getId, id);
        updateWrapper.set(StringUtils.isNotBlank(errandName), Errand::getErrandName, errandName);
        updateWrapper.set(StringUtils.isNotBlank(errandPhone), Errand::getErrandPhone, errandPhone);
        updateWrapper.set(StringUtils.isNotBlank(distributionScope), Errand::getDistributionScope, distributionScope);
        updateWrapper.set(ObjectUtils.isNotEmpty(gender), Errand::getGender, gender);
        updateWrapper.set(ObjectUtils.isNotEmpty(state), Errand::getState, state);
        boolean update = this.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "更新跑腿失败");

        Long userId = errand.getUserId();
        User user = userService.getById(userId);
        if (StringUtils.isNotBlank(errand.getErrandPhone())) {
            user.setPhone(errand.getErrandPhone());
        }
        if ((StringUtils.isNotBlank(errand.getErrandName()))) {
            user.setUsername(errand.getErrandName());
        }

        return true;
    }

    @Override
    public void updateTotal(Errand errand, boolean add) {
        Integer totalOrders = errand.getTotalOrders();
        Integer maxOrders = errand.getMaxOrders();

        if (add) {
            if (totalOrders >= maxOrders) {
                throw new BusinessException(ErrorCode.FORBIDDEN_ERROR, "跑腿员已达到最大接单量，无法接单");
            }
            errand.setTotalOrders(totalOrders + 1);
        } else {
            errand.setTotalOrders(totalOrders - 1);
        }
        boolean update = this.updateById(errand);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "更新跑腿订单量失败");
    }

    @Override
    public ErrandVO getErrandVO(Errand errand) {

        if (errand == null) {
            return null;
        }
        ErrandVO errandVO = new ErrandVO();
        BeanUtils.copyProperties(errand, errandVO);
        ErrandLevel level = errandLevelService.getOne(Wrappers.<ErrandLevel>lambdaQuery()
                .eq(ErrandLevel::getErrandId, errand.getId()));
        errandVO.setLevel(level.getLevel());

        return errandVO;
    }
}




