package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.mapper.OrderDetailsMapper;
import com.bsz.school_send_back_end.model.domain.Dishes;
import com.bsz.school_send_back_end.model.domain.OrderDetails;
import com.bsz.school_send_back_end.model.vo.DishesVO;
import com.bsz.school_send_back_end.model.vo.OrderDetailsVO;
import com.bsz.school_send_back_end.service.DishesService;
import com.bsz.school_send_back_end.service.OrderDetailsService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import jakarta.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xy
 */
@Service
public class OrderDetailsServiceImpl extends ServiceImpl<OrderDetailsMapper, OrderDetails>
        implements OrderDetailsService {

    @Resource
    private DishesService dishesService;

    @Override
    public void validOrderDetail(OrderDetails orderDetails) {
        Long dishesId = orderDetails.getDishesId();
        Integer quantity = orderDetails.getQuantity();
        BigDecimal price = orderDetails.getPrice();
        // 校验菜品是否存在
        Dishes dishes = dishesService.getById(dishesId);
        ThrowUtils.throwIf(dishes == null, ErrorCode.NOT_FOUND_ERROR, "菜品不存在");
        // 有参数则校验
        if (ObjectUtils.allNotNull(dishesId, quantity, price)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "菜品错误");
        }
    }

    @Override
    public OrderDetailsVO getOrderDetailsVO(OrderDetails orderDetails) {
        if (orderDetails == null) {
            return null;
        }
        OrderDetailsVO orderDetailsVO = new OrderDetailsVO();
        BeanUtils.copyProperties(orderDetails, orderDetailsVO);
        // 关联查询菜品信息
        Long dishesId = orderDetails.getDishesId();
        Dishes dishes = null;
        if (dishesId != null && dishesId > 0) {
            dishes = dishesService.getById(dishesId);
        }
        DishesVO dishesVO = dishesService.getDishesVO(dishes);
        orderDetailsVO.setDishesVO(dishesVO);
        return orderDetailsVO;
    }

    @Override
    public List<OrderDetailsVO> getOrderDetailsVO(List<OrderDetails> orderDetailsList) {
        if (CollectionUtils.isEmpty(orderDetailsList)) {
            return new ArrayList<>();
        }
        return orderDetailsList.stream().map(this::getOrderDetailsVO).collect(Collectors.toList());
    }

}




