package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.mapper.SpecificationsDishesMapper;
import com.bsz.school_send_back_end.model.domain.Specifications;
import com.bsz.school_send_back_end.model.domain.SpecificationsDishes;
import com.bsz.school_send_back_end.model.vo.SpecificationsVO;
import com.bsz.school_send_back_end.service.AttributeService;
import com.bsz.school_send_back_end.service.SpecificationsDishesService;
import com.bsz.school_send_back_end.service.SpecificationsService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import jakarta.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author xy
*/
@Service
public class SpecificationsDishesServiceImpl extends ServiceImpl<SpecificationsDishesMapper, SpecificationsDishes>
    implements SpecificationsDishesService {
    @Resource
    private SpecificationsService specificationsService;

    @Resource
    private AttributeService attributeService;

    @Override
    public void deleteByDishesId(Long dishesId) {
        QueryWrapper<SpecificationsDishes> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("dishesId",dishesId);
        long count = this.count(queryWrapper);
        if (count < 1) {
            return;
        }
        boolean remove = this.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);
    }

    @Override
    public Boolean add(Long dishesId, List<Long> specificationsIds) {
        if (CollectionUtils.isEmpty(specificationsIds)) {
           return true;
        }

        for (Long specificationsId: specificationsIds) {
            SpecificationsDishes specificationsDishes = new SpecificationsDishes();
            specificationsDishes.setSpecificationsId(specificationsId);
            specificationsDishes.setDishesId(dishesId);
            boolean save = this.save(specificationsDishes);
            ThrowUtils.throwIf(!save,ErrorCode.OPERATION_ERROR);
        }
        return true;
    }

    @Override
    public Boolean updateByDishesId(Long dishesId, List<Long> specificationsIds) {
        //删除之前的中间表信息
        deleteByDishesId(dishesId);
        //保存更新的中间表信息
        return add(dishesId,specificationsIds);
    }

    @Override
    public List<SpecificationsVO> getSpecificationsAddAttribute(List<SpecificationsDishes> specificationsDishesList) {
        List<SpecificationsVO> specificationsVOList = specificationsDishesList.stream().map(item -> {
            Long specificationsId = item.getSpecificationsId();
            Specifications specifications = specificationsService.getById(specificationsId);
            SpecificationsVO specificationsVO = new SpecificationsVO();
            BeanUtils.copyProperties(specifications, specificationsVO);
            return specificationsVO;
        }).collect(Collectors.toList());
        return attributeService.getBySpecificationsVoId(specificationsVOList);
    }
}




