package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.*;
import com.bsz.school_send_back_end.service.*;
import com.bsz.school_send_back_end.mapper.UserRatingMapper;
import org.springframework.stereotype.Service;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;

/**
* @author xy
*/
@Service
public class UserRatingServiceImpl extends ServiceImpl<UserRatingMapper, UserRating>
    implements UserRatingService{

    @Resource
    private UserService userService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private BusinessService businessService;

    @Resource
    private ErrandService errandService;

    @Override
    public void validUserRating(UserRating userRating, HttpServletRequest request) {

        if (userRating.getRating() < 1 || userRating.getRating() > 5) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "分数不在可选范围内");
        }

        User loginUser = userService.getLoginUser(request);
        if (!loginUser.getId().equals(userRating.getUserId())) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "当前登录用户与评分中的用户不匹配");
        }
        Integer ratedEntityType = userRating.getRatedEntityType();
        Orders orders = ordersService.getOne(Wrappers.<Orders>lambdaQuery().eq(Orders::getId, userRating.getOrderId()));
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        if (ratedEntityType == 0) {
            ThrowUtils.throwIf(!orders.getBusinessId().equals(userRating.getRatedEntityId()) ,ErrorCode.NOT_FOUND_ERROR, "该商家与订单商家不一致");
        }
        else if (ratedEntityType == 1) {
            ThrowUtils.throwIf(!orders.getErrandId().equals(userRating.getRatedEntityId()), ErrorCode.OPERATION_ERROR, "该跑腿与订单跑腿不一致");
        }else {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "分类id不正确");
        }

        long count = this.count(Wrappers.<UserRating>lambdaQuery()
                .eq(UserRating::getOrderId, userRating.getOrderId())
                .eq(UserRating::getRatedEntityType, userRating.getRatedEntityType()));
        ThrowUtils.throwIf(count > 0, ErrorCode.SYSTEM_ERROR, "该订单已评分");
    }

    @Override
    public void validUserRatingByBusiness(Long userRatingId, HttpServletRequest request) {
        LambdaQueryWrapper<UserRating> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserRating::getId, userRatingId);
        UserRating userRating = this.getOne(queryWrapper);
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        if (businessId.equals(userRating.getRatedEntityId())) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR,"该用户评分不属于当前商家");
        }
        if (StringUtils.isBlank(userRating.getReview())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "当前用户没有评论");
        }
        if (StringUtils.isNotBlank(userRating.getBusinessReview())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"当前用户评分已回复");
        }
    }

    @Override
    public List<UserRating> listUserRatingByUser(HttpServletRequest request) {
        Long ratedEntityId = this.authenticationUser(request);

        return this.list(Wrappers.<UserRating>lambdaQuery()
                .eq(UserRating::getRatedEntityId, ratedEntityId));
    }

    @Override
    public int calculateLevel(Double averageScore) {
        if (averageScore >= 4.5) {
            return 5;
        } else if (averageScore >= 4.0) {
            return 4;
        } else if (averageScore >= 3.0) {
            return 3;
        } else if (averageScore >= 2.0) {
            return 2;
        } else {
            return 1;
        }
    }


    private Long authenticationUser (HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        String userRole = loginUser.getUserRole();
        Long ratedEntityId = null;
        if (userRole.equals(UserConstant.BUSINESS_ROLE)) {
            Business business = businessService.getOne(Wrappers.<Business>lambdaQuery()
                    .eq(Business::getUserId, loginUser.getId()));
            ratedEntityId = business.getId();
        } else if (userRole.equals(UserConstant.ERRAND_ROLE)) {
            Errand errand = errandService.getOne(Wrappers.<Errand>lambdaQuery()
                    .eq(Errand::getUserId, loginUser.getId()));
            ratedEntityId = errand.getId();
        }

        return ratedEntityId;
    }

}




