package com.bsz.school_send_back_end.utils;

import com.alibaba.fastjson.JSON;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsRequest;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.profile.DefaultProfile;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;

@Component
@Slf4j
public class AliyunSmsUtils {

    @Value("${aliyun.sms.sms-access-key-id}")
    private String accessKeyId;

    @Value("${aliyun.sms.sms-access-key-secret}")
    private String accessKeySecret;

    @Value("${aliyun.sms.sms-sign-nam}")
    private String signName;

    @Value("${aliyun.sms.sms-template-cod}")
    private String templateCode;

    @Value("${aliyun.sms.sms-endpoint}")
    private String endpoint;

    public boolean sendSms(String phoneNumber, HashMap<String, Object> templateParams) {
        try {
            DefaultProfile profile = DefaultProfile.getProfile("cn-huhehaote", accessKeyId, accessKeySecret);
            DefaultAcsClient client = new DefaultAcsClient(profile);

            SendSmsRequest sendSmsRequest = new SendSmsRequest();
            sendSmsRequest.setPhoneNumbers(phoneNumber);
            sendSmsRequest.setSignName(signName);
            sendSmsRequest.setTemplateCode(templateCode);
            sendSmsRequest.setTemplateParam(JSON.toJSONString(templateParams));

            SendSmsResponse sendSmsResponse = client.getAcsResponse(sendSmsRequest);
            log.info("短信响应: Code={}, Message={}", sendSmsResponse.getCode(), sendSmsResponse.getMessage());

            return "OK".equalsIgnoreCase(sendSmsResponse.getCode());
        } catch (ClientException e) {
            log.error("短信发送异常: {}", e.getErrMsg());
            return false;
        }
    }

}
