package com.bsz.school_send_back_end.utils;




import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.BusinessException;

import jakarta.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * Excel工具类
 *
 * @author xy
 * */
@SuppressWarnings("all")
public class ExcelUtils {

    /**
     * 获取路径
     *
     * @return 当前路径
     */
    public static String getPath() {
        return ExcelUtils.class.getResource("/").getPath();
    }

    /**
     * 创建新文件
     *
     * @param pathName 文件名
     * @return 文件
     */
    public static File createNewFile(String pathName) {
        File file = new File(getPath() + pathName);
        if (file.exists()) {
            file.delete();
        } else {
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
        }
        return file;
    }

    /**
     * 设置响应结果
     *
     * @param response    响应结果对象
     * @param rawFileName 文件名
     */
    public static void setExcelResponseProp(HttpServletResponse response, String rawFileName) throws IOException {
        //设置内容类型
//        response.setContentType("application/vnd.vnd.ms-excel");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        //设置编码格式
        response.setCharacterEncoding("utf-8");
        //设置导出文件名称（避免乱码）
        String fileName = URLEncoder.encode(rawFileName.concat(".xlsx"), "UTF-8");
        // 设置响应头
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName);
    }

    /**
     * Date转String
     *
     * @param date 日期
     * @return 字符串
     */

    public static String dateToString(LocalDateTime date) {
        if (date == null) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR);
        }
        // 使用 DateTimeFormatter 格式化 LocalDateTime
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
        return date.format(formatter);
    }


    /**
     * 转换跑腿订单状态
     * @param errandState
     * @return
     */
    public static String getErrandStateText(Integer errandState) {
        switch (errandState) {
            case 1: return "待抢单";
            case 2: return "已抢单";
            case 3: return "已取货";
            case 4: return "已送达";
            default: return "未知状态";
        }
    }

    /**
     * 转换订单状态
     * @param sate
     * @return
     */
    public static String getStateText(Integer state) {
        switch (state) {
            case 0: return "未支付";
            case 1: return "已支付";
            case 2: return "已退款";
            case 3: return "已取消";
            case 4: return "已出餐";
            case 5: return "已完成";
            default: return "未知状态";
        }
    }

}
